@@ -17,30 +17,59 @@ elif not os.path.exists(BADGER_DB_ROOT):
         f'Badger database root {BADGER_DB_ROOT} created')
 
 
+def maybe_create_routines_db(func):
+
+    def func_safe(*args, **kwargs):
+        db_routine = os.path.join(BADGER_DB_ROOT, 'routines.db')
+
+        con = sqlite3.connect(db_routine)
+        cur = con.cursor()
+
+        cur.execute('create table if not exists routine (name not null primary key, config, savedAt timestamp)')
+
+        con.commit()
+        con.close()
+
+        return func(*args, **kwargs)
+
+    return func_safe
+
+
+def maybe_create_runs_db(func):
+
+    def func_safe(*args, **kwargs):
+        db_run = os.path.join(BADGER_DB_ROOT, 'runs.db')
+
+        con = sqlite3.connect(db_run)
+        cur = con.cursor()
+
+        cur.execute('create table if not exists run (id integer primary key, savedAt timestamp, finishedAt timestamp, routine, filename)')
+
+        con.commit()
+        con.close()
+
+        return func(*args, **kwargs)
+
+    return func_safe
+
+
+@maybe_create_routines_db
 def save_routine(routine):
     db_routine = os.path.join(BADGER_DB_ROOT, 'routines.db')
 
-    # Initialize
-    if not os.path.exists(db_routine):
-        con = sqlite3.connect(db_routine)
-        cur = con.cursor()
-        cur.execute('create table routine (name not null primary key, config, savedAt timestamp)')
-    else:
-        con = sqlite3.connect(db_routine)
-        cur = con.cursor()
+    con = sqlite3.connect(db_routine)
+    cur = con.cursor()
 
     # Insert a record
-    try:
-        cur.execute('insert into routine values (?, ?, ?)',
-                    (routine['name'], yaml.dump(routine, sort_keys=False), datetime.now()))
-    except sqlite3.OperationalError:
-        cur.execute('create table routine (name not null primary key, config, savedAt timestamp)')
-        cur.execute('insert into routine values (?, ?, ?)',
-                    (routine['name'], yaml.dump(routine, sort_keys=False), datetime.now()))
+    cur.execute('insert into routine values (?, ?, ?)',
+                (routine['name'], yaml.dump(routine, sort_keys=False), datetime.now()))
+
     con.commit()
     con.close()
 
 
+@maybe_create_routines_db
+@maybe_create_runs_db
 def remove_routine(name, remove_runs=False):
     db_routine = os.path.join(BADGER_DB_ROOT, 'routines.db')
 
@@ -65,16 +94,14 @@ def remove_routine(name, remove_runs=False):
         con.close()
 
 
+@maybe_create_routines_db
 def load_routine(name):
     db_routine = os.path.join(BADGER_DB_ROOT, 'routines.db')
     con = sqlite3.connect(db_routine)
     cur = con.cursor()
 
-    try:
-        cur.execute('select * from routine where name=:name', {'name': name})
-    except sqlite3.OperationalError:
-        cur.execute('create table routine (name not null primary key, config, savedAt timestamp)')
-        cur.execute('select * from routine where name=:name', {'name': name})
+    cur.execute('select * from routine where name=:name', {'name': name})
+
     records = cur.fetchall()
     con.close()
 
@@ -88,16 +115,14 @@ def load_routine(name):
             f'Multiple routines with name {name} found in the database!')
 
 
+@maybe_create_routines_db
 def list_routine(keyword=''):
     db_routine = os.path.join(BADGER_DB_ROOT, 'routines.db')
     con = sqlite3.connect(db_routine)
     cur = con.cursor()
 
-    try:
-        cur.execute(f'select name, savedAt from routine where name like "%{keyword}%" order by savedAt desc')
-    except sqlite3.OperationalError:
-        cur.execute('create table routine (name not null primary key, config, savedAt timestamp)')
-        cur.execute(f'select name, savedAt from routine where name like "%{keyword}%" order by savedAt desc')
+    cur.execute(f'select name, savedAt from routine where name like "%{keyword}%" order by savedAt desc')
+
     records = cur.fetchall()
     names = [record[0] for record in records]
     timestamps = [record[1] for record in records]
@@ -106,17 +131,12 @@ def list_routine(keyword=''):
     return names, timestamps
 
 
+@maybe_create_runs_db
 def save_run(run):
     db_run = os.path.join(BADGER_DB_ROOT, 'runs.db')
 
-    # Initialize
-    if not os.path.exists(db_run):
-        con = sqlite3.connect(db_run)
-        cur = con.cursor()
-        cur.execute('create table run (id integer primary key, savedAt timestamp, finishedAt timestamp, routine, filename)')
-    else:
-        con = sqlite3.connect(db_run)
-        cur = con.cursor()
+    con = sqlite3.connect(db_run)
+    cur = con.cursor()
 
     # Insert a record
     routine_name = run['routine']['name']
@@ -124,13 +144,9 @@ def save_run(run):
     timestamps = run['data']['timestamp_raw']
     time_start = datetime.fromtimestamp(timestamps[0])
     time_finish = datetime.fromtimestamp(timestamps[-1])
-    try:
-        cur.execute('insert into run values (?, ?, ?, ?, ?)',
-                    (None, time_start, time_finish, routine_name, run_filename))
-    except sqlite3.OperationalError:
-        cur.execute('create table run (id integer primary key, savedAt timestamp, finishedAt timestamp, routine, filename)')
-        cur.execute('insert into run values (?, ?, ?, ?, ?)',
-                    (None, time_start, time_finish, routine_name, run_filename))
+    cur.execute('insert into run values (?, ?, ?, ?, ?)',
+                (None, time_start, time_finish, routine_name, run_filename))
+
     rid = cur.lastrowid
 
     con.commit()
@@ -139,6 +155,7 @@ def save_run(run):
     return rid
 
 
+@maybe_create_runs_db
 def get_runs_by_routine(routine: str):
     db_run = os.path.join(BADGER_DB_ROOT, 'runs.db')
 
@@ -154,6 +171,7 @@ def get_runs_by_routine(routine: str):
     return filenames
 
 
+@maybe_create_runs_db
 def get_runs():
     db_run = os.path.join(BADGER_DB_ROOT, 'runs.db')
 
@@ -162,6 +180,7 @@ def get_runs():
 
     cur.execute(f'select filename from run order by savedAt desc')
     records = cur.fetchall()
+
     con.close()
 
     filenames = [record[0] for record in records]
@@ -169,6 +188,7 @@ def get_runs():
     return filenames
 
 
+@maybe_create_runs_db
 def remove_run_by_filename(name):
     db_run = os.path.join(BADGER_DB_ROOT, 'runs.db')
 
@@ -181,6 +201,7 @@ def remove_run_by_filename(name):
     con.close()
 
 
+@maybe_create_runs_db
 def remove_run_by_id(rid):
     db_run = os.path.join(BADGER_DB_ROOT, 'runs.db')
 
