@@ -1,6 +1,5 @@
 import numpy as np
 import time
-import multiprocessing as mp
 from abc import ABC, abstractmethod
 from typing import List
 from .interface import Interface
@@ -33,9 +32,8 @@ class Environment(ABC):
 
     # Get the default params of the environment
     @staticmethod
-    @abstractmethod
     def get_default_params() -> dict:
-        pass
+        return None
 
     # Get current variable
     # Unsafe version (var won't be checked beforehand)
@@ -60,6 +58,12 @@ class Environment(ABC):
     def vars_changed(self, vars: List[str], values: list):
         pass
 
+    # Get current system states
+    # If return is not None, the states would be saved at the start of each run
+    # Should return a dict if not None
+    def get_system_states(self):
+        return None
+
     # Get observation
     # Unsafe version
     @abstractmethod
