@@ -1,10 +1,12 @@
 import numpy as np
-from PyQt5.QtWidgets import QVBoxLayout, QHBoxLayout, QWidget, QPushButton, QMessageBox
+from PyQt5.QtWidgets import QVBoxLayout, QHBoxLayout, QWidget, QPushButton
+from PyQt5.QtWidgets import QMessageBox, QComboBox, QLabel, QStyledItemDelegate
 from PyQt5.QtCore import pyqtSignal, QThreadPool, Qt
 from PyQt5.QtGui import QFont
 import pyqtgraph as pg
 from ..components.routine_runner import BadgerRoutineRunner
 # from ..utils import AURORA_PALETTE, FROST_PALETTE
+from ...utils import norm
 from ...logbook import send_to_logbook, BADGER_LOGBOOK_ROOT
 from ...archive import archive_run, BADGER_ARCHIVE_ROOT
 
@@ -23,6 +25,10 @@ class BadgerOptMonitor(QWidget):
                           for d in self.routine['config']['objectives']]
         self.var_names = [next(iter(d))
                           for d in self.routine['config']['variables']]
+        # For plot type switching
+        self.x_plot_type = 0  # 0: raw, 1: normalized
+        self.vranges = np.array([d[next(iter(d))]
+                                 for d in routine['config']['variables']])
         if self.routine['config']['constraints']:
             self.con_names = [next(iter(d))
                               for d in self.routine['config']['constraints']]
@@ -40,6 +46,17 @@ class BadgerOptMonitor(QWidget):
 
         vbox = QVBoxLayout(self)
 
+        # Config bar
+        config_bar = QWidget()
+        hbox_config = QHBoxLayout(config_bar)
+        hbox_config.setContentsMargins(0, 0, 0, 0)
+        label = QLabel('Evaluation History (X) Plot Type')
+        self.cb_plot = cb_plot = QComboBox()
+        cb_plot.setItemDelegate(QStyledItemDelegate())
+        cb_plot.addItems(['Raw', 'Normalized'])
+        hbox_config.addWidget(label)
+        hbox_config.addWidget(cb_plot, 1)
+
         # Set up the monitor
         # Don't set show=True or there will be a blank window flashing once
         self.monitor = monitor = pg.GraphicsLayoutWidget()
@@ -182,6 +199,7 @@ class BadgerOptMonitor(QWidget):
         hbox_action.addWidget(btn_ctrl)
         hbox_action.addWidget(btn_stop)
 
+        vbox.addWidget(config_bar)
         vbox.addWidget(monitor)
         vbox.addWidget(action_bar)
 
@@ -226,6 +244,9 @@ class BadgerOptMonitor(QWidget):
         self.btn_ctrl.clicked.connect(self.ctrl_routine)
         self.btn_stop.clicked.connect(self.stop_routine)
 
+        # Visualization
+        self.cb_plot.currentIndexChanged.connect(self.select_x_plot_type)
+
     def center(self):
         pass
 
@@ -233,6 +254,30 @@ class BadgerOptMonitor(QWidget):
         self.running = True  # if a routine runner is working
         self.thread_pool.start(self.routine_runner)
 
+    def is_critical(self, cons):
+        if not self.con_names:
+            return False, None
+
+        constraints = self.routine['config']['constraints']
+        for i, con_dict in enumerate(constraints):
+            name = self.con_names[i]
+            if len(con_dict[name]) != 3:
+                continue
+
+            value = cons[i]
+            relation, thres = con_dict[name][:2]
+            if relation == 'GREATER_THAN':
+                if value <= thres:
+                    return True, f'{name} (current value: {value:.4f}) is less than {thres}!'
+            elif relation == 'LESS_THAN':
+                if value >= thres:
+                    return True, f'{name} (current value: {value:.4f}) is greater than {thres}!'
+            else:
+                if value != thres:
+                    return True, f'{name} (current value: {value:.4f}) is not equal to {thres}!'
+
+        return False, None
+
     def update(self, vars, objs, cons):
         self.vars.append(vars)
         self.objs.append(objs)
@@ -245,8 +290,27 @@ class BadgerOptMonitor(QWidget):
             for i in range(len(self.con_names)):
                 self.curves_con[i].setData(np.array(self.cons)[:, i])
 
+        if self.x_plot_type:
+            _vars = norm(np.array(self.vars), self.vranges[:, 0], self.vranges[:, 1])
+        else:
+            _vars = np.array(self.vars)
         for i in range(len(self.var_names)):
-            self.curves_var[i].setData(np.array(self.vars)[:, i])
+            self.curves_var[i].setData(_vars[:, i])
+
+        # Check critical condition
+        critical, msg = self.is_critical(cons)
+        if not critical:
+            return
+
+        self.sig_pause.emit(True)
+        self.btn_ctrl.setText('Resume')
+
+        reply = QMessageBox.warning(self,
+                                    'Run Paused',
+                                    f'Critical constraint was violated: {msg}\nTerminate the run?',
+                                    QMessageBox.Yes | QMessageBox.No, QMessageBox.Yes)
+        if reply == QMessageBox.Yes:
+            self.sig_stop.emit()
 
     def env_ready(self, init_vars):
         self.env = self.routine_runner.env
@@ -324,9 +388,9 @@ class BadgerOptMonitor(QWidget):
         if reply != QMessageBox.Yes:
             return
 
-        current_vars = self.env.get_vars(self.var_names)
-        self.env.set_vars(self.var_names, self.init_vars)
-        after_vars = self.env.get_vars(self.var_names)
+        current_vars = self.env._get_vars(self.var_names)
+        self.env._set_vars(self.var_names, self.init_vars)
+        after_vars = self.env._get_vars(self.var_names)
         QMessageBox.information(
             self, 'Reset Environment', f'Env vars {current_vars} -> {after_vars}')
 
@@ -351,12 +415,21 @@ class BadgerOptMonitor(QWidget):
         if reply != QMessageBox.Yes:
             return
 
-        self.env.set_vars(self.var_names, solution)
+        self.env._set_vars(self.var_names, solution)
         # center around the inspector
         self.plot_var.setXRange(idx - 3, idx + 3)
         # QMessageBox.information(
         #     self, 'Set Environment', f'Env vars have been set to {solution}')
 
+    def select_x_plot_type(self, i):
+        self.x_plot_type = i
+        if i:
+            _vars = norm(np.array(self.vars), self.vranges[:, 0], self.vranges[:, 1])
+        else:
+            _vars = np.array(self.vars)
+        for i in range(len(self.var_names)):
+            self.curves_var[i].setData(_vars[:, i])
+
     def _make_label(self):
         try:
             df = self.routine_runner.data
