@@ -4,14 +4,15 @@ from PyQt5.QtCore import QSize
 import sqlite3
 from coolname import generate_slug
 from ...factory import list_algo, list_env, get_algo, get_env
-from ...utils import ystring, load_config, config_list_to_dict, normalize_routine, instantiate_env
+from ...utils import ystring, load_config, config_list_to_dict
+from ...core import normalize_routine, instantiate_env, list_scaling_func, get_scaling_default_params
 from ...db import save_routine, remove_routine
 from .variable_item import variable_item
 from .objective_item import objective_item
 from .constraint_item import constraint_item
+from .state_item import state_item
 from .algo_cbox import BadgerAlgoBox
 from .env_cbox import BadgerEnvBox
-from .config_cbox import BadgerConfigBox
 from ..windows.review_dialog import BadgerReviewDialog
 from ..windows.var_dialog import BadgerVariableDialog
 from ..windows.edit_script_dialog import BadgerEditScriptDialog
@@ -33,6 +34,7 @@ class BadgerRoutinePage(QWidget):
         self.env = None
         self.routine = None
         self.script = ''
+        self.scaling_functions = list_scaling_func()
 
         self.init_ui()
         self.config_logic()
@@ -54,41 +56,42 @@ class BadgerRoutinePage(QWidget):
         vbox.addWidget(group_meta)
 
         # Algo box
-        self.algo_box = BadgerAlgoBox(self.algos)
+        self.algo_box = BadgerAlgoBox(self.algos, self.scaling_functions)
+        self.algo_box.expand()  # expand the box initially
         vbox.addWidget(self.algo_box)
 
         # Env box
         self.env_box = BadgerEnvBox(self.envs)
+        self.env_box.expand()  # expand the box initially
         vbox.addWidget(self.env_box)
 
-        # Configs box
-        self.configs_box = BadgerConfigBox()
-        vbox.addWidget(self.configs_box)
-
         vbox.addStretch()
 
     def config_logic(self):
         self.algo_box.cb.currentIndexChanged.connect(self.select_algo)
         self.algo_box.check_use_script.stateChanged.connect(self.toggle_use_script)
         self.algo_box.btn_edit_script.clicked.connect(self.edit_script)
+        self.algo_box.cb_scaling.currentIndexChanged.connect(self.select_scaling_func)
         self.env_box.cb.currentIndexChanged.connect(self.select_env)
         self.env_box.btn_env_play.clicked.connect(self.open_playground)
-        self.configs_box.btn_all_var.clicked.connect(self.check_all_var)
-        self.configs_box.btn_un_all_var.clicked.connect(self.uncheck_all_var)
-        self.configs_box.btn_add_var.clicked.connect(self.add_var)
-        self.configs_box.btn_all_obj.clicked.connect(self.check_all_obj)
-        self.configs_box.btn_un_all_obj.clicked.connect(self.uncheck_all_obj)
-        self.configs_box.check_only_var.stateChanged.connect(self.toggle_check_only_var)
-        self.configs_box.check_only_obj.stateChanged.connect(self.toggle_check_only_obj)
-        self.configs_box.btn_add_con.clicked.connect(self.add_constraint)
+        self.env_box.btn_all_var.clicked.connect(self.check_all_var)
+        self.env_box.btn_un_all_var.clicked.connect(self.uncheck_all_var)
+        self.env_box.btn_add_var.clicked.connect(self.add_var)
+        self.env_box.btn_all_obj.clicked.connect(self.check_all_obj)
+        self.env_box.btn_un_all_obj.clicked.connect(self.uncheck_all_obj)
+        self.env_box.check_only_var.stateChanged.connect(self.toggle_check_only_var)
+        self.env_box.check_only_obj.stateChanged.connect(self.toggle_check_only_obj)
+        self.env_box.btn_add_con.clicked.connect(self.add_constraint)
+        self.env_box.btn_add_sta.clicked.connect(self.add_state)
 
     def refresh_ui(self, routine):
         self.routine = routine  # save routine for future reference
 
         self.algos = list_algo()
         self.envs = list_env()
-        # Clean up the constraints list
-        self.configs_box.list_con.clear()
+        # Clean up the constraints/states list
+        self.env_box.list_con.clear()
+        self.env_box.list_sta.clear()
 
         if routine is None:
             # Reset the algo and env configs
@@ -96,8 +99,8 @@ class BadgerRoutinePage(QWidget):
             self.env_box.cb.setCurrentIndex(-1)
 
             # Reset the routine configs check box status
-            self.configs_box.check_only_var.setChecked(False)
-            self.configs_box.check_only_obj.setChecked(False)
+            self.env_box.check_only_var.setChecked(False)
+            self.env_box.check_only_obj.setChecked(False)
 
             # Reset the save settings
             name = generate_slug(2)
@@ -115,6 +118,19 @@ class BadgerRoutinePage(QWidget):
             self.script = routine['config']['script']
         except KeyError:
             self.script = None
+        try:
+            name_scaling = routine['config']['domain_scaling']['func']
+            params_scaling = routine['config']['domain_scaling'].copy()
+            del params_scaling['func']
+        except:
+            name_scaling = None
+            params_scaling = None
+        try:
+            idx_scaling = self.scaling_functions.index(name_scaling)
+        except ValueError:
+            idx_scaling = -1
+        self.algo_box.cb_scaling.setCurrentIndex(idx_scaling)
+        self.algo_box.edit_scaling.setPlainText(ystring(params_scaling))
 
         name_env = routine['env']
         idx_env = self.envs.index(name_env)
@@ -122,11 +138,11 @@ class BadgerRoutinePage(QWidget):
         self.env_box.edit.setPlainText(ystring(routine['env_params']))
 
         # Config the vocs panel
-        self.configs_box.check_only_var.setChecked(True)
+        self.env_box.check_only_var.setChecked(True)
         variables = [next(iter(v)) for v in routine['config']['variables']]
-        for i in range(self.configs_box.list_var.count()):
-            item = self.configs_box.list_var.item(i)
-            item_widget = self.configs_box.list_var.itemWidget(item)
+        for i in range(self.env_box.list_var.count()):
+            item = self.env_box.list_var.item(i)
+            item_widget = self.env_box.list_var.itemWidget(item)
             var_name = item_widget.check_name.text()
             try:
                 idx = variables.index(var_name)
@@ -137,11 +153,11 @@ class BadgerRoutinePage(QWidget):
             except ValueError:
                 item_widget.check_name.setChecked(False)
 
-        self.configs_box.check_only_obj.setChecked(True)
+        self.env_box.check_only_obj.setChecked(True)
         objectives = [next(iter(v)) for v in routine['config']['objectives']]
-        for i in range(self.configs_box.list_obj.count()):
-            item = self.configs_box.list_obj.item(i)
-            item_widget = self.configs_box.list_obj.itemWidget(item)
+        for i in range(self.env_box.list_obj.count()):
+            item = self.env_box.list_obj.item(i)
+            item_widget = self.env_box.list_obj.itemWidget(item)
             obj_name = item_widget.check_name.text()
             try:
                 idx = objectives.index(obj_name)
@@ -165,6 +181,14 @@ class BadgerRoutinePage(QWidget):
                             'EQUAL_TO'].index(relation)
                 self.add_constraint(name, relation, thres, critical)
 
+        try:
+            config_states = routine['config']['states']
+        except KeyError:
+            config_states = None
+        if config_states is not None:
+            for name_sta in config_states:
+                self.add_state(name_sta)
+
         # Config the save settings
         name = routine['name']
         self.edit_save.setPlaceholderText(generate_slug(2))
@@ -179,6 +203,7 @@ class BadgerRoutinePage(QWidget):
 
         if i == -1:
             self.algo_box.edit.setPlainText('')
+            self.algo_box.cb_scaling.setCurrentIndex(-1)
             return
 
         name = self.algos[i]
@@ -241,17 +266,31 @@ class BadgerRoutinePage(QWidget):
         except Exception as e:
             QMessageBox.warning(self, 'Invalid script!', str(e))
 
+    def select_scaling_func(self, i):
+        if i == -1:
+            self.algo_box.edit_scaling.setPlainText('')
+            return
+
+        try:
+            name = self.scaling_functions[i]
+            default_params = get_scaling_default_params(name)
+            self.algo_box.edit_scaling.setPlainText(ystring(default_params))
+        except Exception as e:
+            self.algo_box.cb.setCurrentIndex(-1)
+            return QMessageBox.critical(self, 'Error!', str(e))
+
     def select_env(self, i):
         if i == -1:
             self.env_box.edit.setPlainText('')
-            self.configs_box.list_var.clear()
-            self.configs_box.dict_var.clear()
-            self.configs_box.list_obj.clear()
-            self.configs_box.dict_obj.clear()
+            self.env_box.list_var.clear()
+            self.env_box.dict_var.clear()
+            self.env_box.list_obj.clear()
+            self.env_box.dict_obj.clear()
             self.configs = None
             self.env = None
-            self.configs_box.btn_add_con.setDisabled(True)
-            self.configs_box.btn_add_var.setDisabled(True)
+            self.env_box.btn_add_con.setDisabled(True)
+            self.env_box.btn_add_sta.setDisabled(True)
+            self.env_box.btn_add_var.setDisabled(True)
             self.routine = None
             return
 
@@ -260,22 +299,24 @@ class BadgerRoutinePage(QWidget):
             env, configs = get_env(name)
             self.configs = configs
             self.env = env
-            self.configs_box.btn_add_con.setDisabled(False)
-            self.configs_box.btn_add_var.setDisabled(False)
+            self.env_box.btn_add_con.setDisabled(False)
+            self.env_box.btn_add_sta.setDisabled(False)
+            self.env_box.btn_add_var.setDisabled(False)
             if self.algo_box.check_use_script.isChecked():
                 self.refresh_params_algo()
         except Exception as e:
             self.configs = None
             self.env = None
             self.env_box.cb.setCurrentIndex(-1)
-            self.configs_box.btn_add_con.setDisabled(True)
-            self.configs_box.btn_add_var.setDisabled(True)
+            self.env_box.btn_add_con.setDisabled(True)
+            self.env_box.btn_add_sta.setDisabled(True)
+            self.env_box.btn_add_var.setDisabled(True)
             self.routine = None
             return QMessageBox.critical(self, 'Error!', str(e))
 
         self.env_box.edit.setPlainText(ystring(configs['params']))
 
-        self.configs_box.list_var.clear()
+        self.env_box.list_var.clear()
         vars_env = configs['variables']
         vars_combine = [*vars_env]
         if self.routine:  # check for the temp variables in vocs
@@ -293,39 +334,40 @@ class BadgerRoutinePage(QWidget):
         for var in vars_combine:
             name = next(iter(var))
             vrange = var[name]
-            item = QListWidgetItem(self.configs_box.list_var)
+            item = QListWidgetItem(self.env_box.list_var)
             var_item = variable_item(name, vrange, self.toggle_var)
             item.setSizeHint(var_item.sizeHint())
-            self.configs_box.list_var.addItem(item)
-            self.configs_box.list_var.setItemWidget(item, var_item)
-            self.configs_box.dict_var[name] = item
+            self.env_box.list_var.addItem(item)
+            self.env_box.list_var.setItemWidget(item, var_item)
+            self.env_box.dict_var[name] = item
 
-        self.configs_box.list_obj.clear()
+        self.env_box.list_obj.clear()
         for obj in configs['observations']:
-            item = QListWidgetItem(self.configs_box.list_obj)
+            item = QListWidgetItem(self.env_box.list_obj)
             obj_item = objective_item(obj, 0, self.toggle_obj)
             item.setSizeHint(obj_item.sizeHint())
-            self.configs_box.list_obj.addItem(item)
-            self.configs_box.list_obj.setItemWidget(item, obj_item)
-            self.configs_box.dict_obj[obj] = item
+            self.env_box.list_obj.addItem(item)
+            self.env_box.list_obj.setItemWidget(item, obj_item)
+            self.env_box.dict_obj[obj] = item
 
-        self.configs_box.list_con.clear()
-        self.configs_box.fit_content()
+        self.env_box.list_con.clear()
+        self.env_box.list_sta.clear()
+        self.env_box.fit_content()
         self.routine = None
 
     def open_playground(self):
         pass
 
     def check_all_var(self):
-        for i in range(self.configs_box.list_var.count()):
-            item = self.configs_box.list_var.item(i)
-            item_widget = self.configs_box.list_var.itemWidget(item)
+        for i in range(self.env_box.list_var.count()):
+            item = self.env_box.list_var.item(i)
+            item_widget = self.env_box.list_var.itemWidget(item)
             item_widget.check_name.setChecked(True)
 
     def uncheck_all_var(self):
-        for i in range(self.configs_box.list_var.count()):
-            item = self.configs_box.list_var.item(i)
-            item_widget = self.configs_box.list_var.itemWidget(item)
+        for i in range(self.env_box.list_var.count()):
+            item = self.env_box.list_var.item(i)
+            item_widget = self.env_box.list_var.itemWidget(item)
             item_widget.check_name.setChecked(False)
 
     def add_var(self):
@@ -347,7 +389,7 @@ class BadgerRoutinePage(QWidget):
         # Check if already in the list
         ok = False
         try:
-            self.configs_box.dict_var[name]
+            self.env_box.dict_var[name]
         except:
             ok = True
 
@@ -355,107 +397,122 @@ class BadgerRoutinePage(QWidget):
             QMessageBox.warning(self, 'Variable already exists!', f'Variable {name} already exists!')
             return 1
 
-        item = QListWidgetItem(self.configs_box.list_var)
+        item = QListWidgetItem(self.env_box.list_var)
         var_item = variable_item(name, [min, max], self.toggle_var)
         var_item.check_name.setChecked(True)
         item.setSizeHint(var_item.sizeHint())
-        self.configs_box.list_var.addItem(item)
-        self.configs_box.list_var.setItemWidget(item, var_item)
-        self.configs_box.dict_var[name] = item
-        self.configs_box.fit_content()
+        self.env_box.list_var.addItem(item)
+        self.env_box.list_var.setItemWidget(item, var_item)
+        self.env_box.dict_var[name] = item
+        self.env_box.fit_content()
 
         return 0
 
     def check_all_obj(self):
-        for i in range(self.configs_box.list_obj.count()):
-            item = self.configs_box.list_obj.item(i)
-            item_widget = self.configs_box.list_obj.itemWidget(item)
+        for i in range(self.env_box.list_obj.count()):
+            item = self.env_box.list_obj.item(i)
+            item_widget = self.env_box.list_obj.itemWidget(item)
             item_widget.check_name.setChecked(True)
 
     def uncheck_all_obj(self):
-        for i in range(self.configs_box.list_obj.count()):
-            item = self.configs_box.list_obj.item(i)
-            item_widget = self.configs_box.list_obj.itemWidget(item)
+        for i in range(self.env_box.list_obj.count()):
+            item = self.env_box.list_obj.item(i)
+            item_widget = self.env_box.list_obj.itemWidget(item)
             item_widget.check_name.setChecked(False)
 
     def toggle_check_only_var(self):
-        if self.configs_box.check_only_var.isChecked():
-            for i in range(self.configs_box.list_var.count()):
-                item = self.configs_box.list_var.item(i)
-                item_widget = self.configs_box.list_var.itemWidget(item)
+        if self.env_box.check_only_var.isChecked():
+            for i in range(self.env_box.list_var.count()):
+                item = self.env_box.list_var.item(i)
+                item_widget = self.env_box.list_var.itemWidget(item)
                 if not item_widget.check_name.isChecked():
                     item_widget.hide()
                     item.setSizeHint(QSize(0, 0))
         else:
-            for i in range(self.configs_box.list_var.count()):
-                item = self.configs_box.list_var.item(i)
-                item_widget = self.configs_box.list_var.itemWidget(item)
+            for i in range(self.env_box.list_var.count()):
+                item = self.env_box.list_var.item(i)
+                item_widget = self.env_box.list_var.itemWidget(item)
                 item_widget.show()
                 item.setSizeHint(item_widget.sizeHint())
-        self.configs_box.fit_content()
+        self.env_box.fit_content()
 
     def toggle_var(self, name):
-        if self.configs_box.check_only_var.isChecked():
-            item = self.configs_box.dict_var[name]
-            item_widget = self.configs_box.list_var.itemWidget(item)
+        if self.env_box.check_only_var.isChecked():
+            item = self.env_box.dict_var[name]
+            item_widget = self.env_box.list_var.itemWidget(item)
             if item_widget.check_name.isChecked():
                 item_widget.show()
                 item.setSizeHint(item_widget.sizeHint())
             else:
                 item_widget.hide()
                 item.setSizeHint(QSize(0, 0))
-            self.configs_box.fit_content()
+            self.env_box.fit_content()
 
     def toggle_check_only_obj(self):
-        if self.configs_box.check_only_obj.isChecked():
-            for i in range(self.configs_box.list_obj.count()):
-                item = self.configs_box.list_obj.item(i)
-                item_widget = self.configs_box.list_obj.itemWidget(item)
+        if self.env_box.check_only_obj.isChecked():
+            for i in range(self.env_box.list_obj.count()):
+                item = self.env_box.list_obj.item(i)
+                item_widget = self.env_box.list_obj.itemWidget(item)
                 if not item_widget.check_name.isChecked():
                     item_widget.hide()
                     item.setSizeHint(QSize(0, 0))
         else:
-            for i in range(self.configs_box.list_obj.count()):
-                item = self.configs_box.list_obj.item(i)
-                item_widget = self.configs_box.list_obj.itemWidget(item)
+            for i in range(self.env_box.list_obj.count()):
+                item = self.env_box.list_obj.item(i)
+                item_widget = self.env_box.list_obj.itemWidget(item)
                 item_widget.show()
                 item.setSizeHint(item_widget.sizeHint())
-        self.configs_box.fit_content()
+        self.env_box.fit_content()
 
     def toggle_obj(self, name):
-        if self.configs_box.check_only_obj.isChecked():
-            item = self.configs_box.dict_obj[name]
-            item_widget = self.configs_box.list_obj.itemWidget(item)
+        if self.env_box.check_only_obj.isChecked():
+            item = self.env_box.dict_obj[name]
+            item_widget = self.env_box.list_obj.itemWidget(item)
             if item_widget.check_name.isChecked():
                 item_widget.show()
                 item.setSizeHint(item_widget.sizeHint())
             else:
                 item_widget.hide()
                 item.setSizeHint(QSize(0, 0))
-            self.configs_box.fit_content()
+            self.env_box.fit_content()
 
     def add_constraint(self, name=None, relation=0, threshold=0, critical=False):
         if self.configs is None:
             return
 
         options = self.configs['observations']
-        item = QListWidgetItem(self.configs_box.list_con)
+        item = QListWidgetItem(self.env_box.list_con)
         con_item = constraint_item(options,
-                                   lambda: self.configs_box.list_con.takeItem(
-                                       self.configs_box.list_con.row(item)),
+                                   lambda: self.env_box.list_con.takeItem(
+                                       self.env_box.list_con.row(item)),
                                    name, relation, threshold, critical)
         item.setSizeHint(con_item.sizeHint())
-        self.configs_box.list_con.addItem(item)
-        self.configs_box.list_con.setItemWidget(item, con_item)
-        # self.configs_box.dict_con[''] = item
-        self.configs_box.fit_content()
+        self.env_box.list_con.addItem(item)
+        self.env_box.list_con.setItemWidget(item, con_item)
+        # self.env_box.dict_con[''] = item
+        self.env_box.fit_content()
+
+    def add_state(self, name=None):
+        if self.configs is None:
+            return
+
+        var_names = [next(iter(d)) for d in self.configs['variables']]
+        options = self.configs['observations'] + var_names
+        item = QListWidgetItem(self.env_box.list_sta)
+        sta_item = state_item(options,
+                              lambda: self.env_box.list_sta.takeItem(
+                                  self.env_box.list_sta.row(item)), name)
+        item.setSizeHint(sta_item.sizeHint())
+        self.env_box.list_sta.addItem(item)
+        self.env_box.list_sta.setItemWidget(item, sta_item)
+        self.env_box.fit_content()
 
     def _compose_vocs(self):
         # Compose the VOCS settings
         variables = []
-        for i in range(self.configs_box.list_var.count()):
-            item = self.configs_box.list_var.item(i)
-            item_widget = self.configs_box.list_var.itemWidget(item)
+        for i in range(self.env_box.list_var.count()):
+            item = self.env_box.list_var.item(i)
+            item_widget = self.env_box.list_var.itemWidget(item)
             if item_widget.check_name.isChecked():
                 var_name = item_widget.check_name.text()
                 lb = item_widget.sb_lower.sb.value()
@@ -465,9 +522,9 @@ class BadgerRoutinePage(QWidget):
                 variables.append(_dict)
 
         objectives = []
-        for i in range(self.configs_box.list_obj.count()):
-            item = self.configs_box.list_obj.item(i)
-            item_widget = self.configs_box.list_obj.itemWidget(item)
+        for i in range(self.env_box.list_obj.count()):
+            item = self.env_box.list_obj.item(i)
+            item_widget = self.env_box.list_obj.itemWidget(item)
             if item_widget.check_name.isChecked():
                 obj_name = item_widget.check_name.text()
                 rule = item_widget.cb_rule.currentText()
@@ -476,9 +533,9 @@ class BadgerRoutinePage(QWidget):
                 objectives.append(_dict)
 
         constraints = []
-        for i in range(self.configs_box.list_con.count()):
-            item = self.configs_box.list_con.item(i)
-            item_widget = self.configs_box.list_con.itemWidget(item)
+        for i in range(self.env_box.list_con.count()):
+            item = self.env_box.list_con.item(i)
+            item_widget = self.env_box.list_con.itemWidget(item)
             critical = item_widget.check_crit.isChecked()
             con_name = item_widget.cb_obs.currentText()
             relation = CONS_RELATION_DICT[item_widget.cb_rel.currentText()]
@@ -489,10 +546,18 @@ class BadgerRoutinePage(QWidget):
                 _dict[con_name].append('CRITICAL')
             constraints.append(_dict)
 
+        states = []
+        for i in range(self.env_box.list_sta.count()):
+            item = self.env_box.list_sta.item(i)
+            item_widget = self.env_box.list_sta.itemWidget(item)
+            sta_name = item_widget.cb_sta.currentText()
+            states.append(sta_name)
+
         vocs = {
             'variables': variables,
             'objectives': objectives,
             'constraints': constraints,
+            'states': states,
         }
 
         return vocs
@@ -508,9 +573,9 @@ class BadgerRoutinePage(QWidget):
         env_params = load_config(self.env_box.edit.toPlainText())
 
         variables = []
-        for i in range(self.configs_box.list_var.count()):
-            item = self.configs_box.list_var.item(i)
-            item_widget = self.configs_box.list_var.itemWidget(item)
+        for i in range(self.env_box.list_var.count()):
+            item = self.env_box.list_var.item(i)
+            item_widget = self.env_box.list_var.itemWidget(item)
             if item_widget.check_name.isChecked():
                 var_name = item_widget.check_name.text()
                 lb = item_widget.sb_lower.sb.value()
@@ -520,9 +585,9 @@ class BadgerRoutinePage(QWidget):
                 variables.append(_dict)
 
         objectives = []
-        for i in range(self.configs_box.list_obj.count()):
-            item = self.configs_box.list_obj.item(i)
-            item_widget = self.configs_box.list_obj.itemWidget(item)
+        for i in range(self.env_box.list_obj.count()):
+            item = self.env_box.list_obj.item(i)
+            item_widget = self.env_box.list_obj.itemWidget(item)
             if item_widget.check_name.isChecked():
                 obj_name = item_widget.check_name.text()
                 rule = item_widget.cb_rule.currentText()
@@ -531,9 +596,9 @@ class BadgerRoutinePage(QWidget):
                 objectives.append(_dict)
 
         constraints = []
-        for i in range(self.configs_box.list_con.count()):
-            item = self.configs_box.list_con.item(i)
-            item_widget = self.configs_box.list_con.itemWidget(item)
+        for i in range(self.env_box.list_con.count()):
+            item = self.env_box.list_con.item(i)
+            item_widget = self.env_box.list_con.itemWidget(item)
             critical = item_widget.check_crit.isChecked()
             con_name = item_widget.cb_obs.currentText()
             relation = CONS_RELATION_DICT[item_widget.cb_rel.currentText()]
@@ -544,10 +609,26 @@ class BadgerRoutinePage(QWidget):
                 _dict[con_name].append('CRITICAL')
             constraints.append(_dict)
 
+        states = []
+        for i in range(self.env_box.list_sta.count()):
+            item = self.env_box.list_sta.item(i)
+            item_widget = self.env_box.list_sta.itemWidget(item)
+            sta_name = item_widget.cb_sta.currentText()
+            states.append(sta_name)
+
+        try:
+            scaling = self.algo_box.cb_scaling.currentText()
+            scaling_params = load_config(self.algo_box.edit_scaling.toPlainText())
+            domain_scaling = {'func': scaling, **scaling_params}
+        except:
+            domain_scaling = None
+
         configs = {
             'variables': variables,
             'objectives': objectives,
             'constraints': constraints,
+            'states': states,
+            'domain_scaling': domain_scaling,
         }
         if self.algo_box.check_use_script.isChecked():
             configs['script'] = self.script
