@@ -26,6 +26,55 @@ QPushButton
 }
 '''
 
+stylesheet_log = '''
+QPushButton:hover:pressed
+{
+    background-color: #5C8899;
+}
+QPushButton:hover
+{
+    background-color: #72A4B4;
+}
+QPushButton
+{
+    background-color: #88C0D0;
+    color: #000000;
+}
+'''
+
+stylesheet_run = '''
+QPushButton:hover:pressed
+{
+    background-color: #92D38C;
+}
+QPushButton:hover
+{
+    background-color: #6EC566;
+}
+QPushButton
+{
+    background-color: #4AB640;
+    color: #000000;
+}
+'''
+
+stylesheet_stop = '''
+QPushButton:hover:pressed
+{
+    background-color: #ed9c33;
+}
+QPushButton:hover
+{
+    background-color: #eb8f1a;
+}
+QPushButton
+{
+    background-color: #E98300;
+    color: #000000;
+}
+'''
+
+
 class BadgerOptMonitor(QWidget):
     sig_pause = pyqtSignal(bool)  # True: pause, False: resume
     sig_stop = pyqtSignal()
@@ -33,7 +82,7 @@ class BadgerOptMonitor(QWidget):
     sig_new_run = pyqtSignal()
     sig_run_name = pyqtSignal(str)  # filename of the new run
     sig_inspect = pyqtSignal(int)  # index of the inspector
-    sig_progress = pyqtSignal(list, list, list)  # new evaluated solution
+    sig_progress = pyqtSignal(list, list, list, list)  # new evaluated solution
     sig_del = pyqtSignal()
 
     def __init__(self):
@@ -49,15 +98,18 @@ class BadgerOptMonitor(QWidget):
         self.var_names = []
         self.obj_names = []
         self.con_names = []
+        self.sta_names = []
         self.vranges = []
         # Curves in the monitor
         self.curves_var = []
         self.curves_obj = []
         self.curves_con = []
+        self.curves_sta = []
         # Data to be visualized
         self.vars = []
         self.objs = []
         self.cons = []
+        self.stas = []
         self.ts = []
         # Run optimization
         self.thread_pool = None
@@ -65,6 +117,8 @@ class BadgerOptMonitor(QWidget):
         self.running = False
         # Analysis tool for history runs
         self.pf = None
+        # Fix the auto range issue
+        self.eval_count = 0
 
         self.init_ui()
         self.config_logic()
@@ -78,7 +132,7 @@ class BadgerOptMonitor(QWidget):
         # Config bar
         config_bar = QWidget()
         hbox_config = QHBoxLayout(config_bar)
-        hbox_config.setContentsMargins(0, 0, 0, 0)
+        hbox_config.setContentsMargins(8, 0, 8, 0)
         label = QLabel('Evaluation History Plot Type')
         label_x = QLabel('X Axis')
         self.cb_plot_x = cb_plot_x = QComboBox()
@@ -114,6 +168,7 @@ class BadgerOptMonitor(QWidget):
         leg_obj.setBrush((50, 50, 100, 200))
 
         monitor.nextRow()  # leave space for the cons plot
+        monitor.nextRow()  # leave space for the stas plot
         monitor.nextRow()
 
         self.plot_var = plot_var = monitor.addPlot(
@@ -141,6 +196,12 @@ class BadgerOptMonitor(QWidget):
                                            'color': (200, 200, 100),
                                            'fill': (200, 200, 200, 50),
                                            'movable': True})
+        self.ins_sta = pg.InfiniteLine(movable=True, angle=90, label=None,
+                                       labelOpts={
+                                           'position': 0.1,
+                                           'color': (200, 200, 100),
+                                           'fill': (200, 200, 200, 50),
+                                           'movable': True})
         self.ins_var = pg.InfiniteLine(movable=True, angle=90, label=None,
                                        labelOpts={
                                            'position': 0.1,
@@ -154,7 +215,7 @@ class BadgerOptMonitor(QWidget):
         action_bar = QWidget()
         # action_bar.hide()
         hbox_action = QHBoxLayout(action_bar)
-        hbox_action.setContentsMargins(0, 0, 0, 0)
+        hbox_action.setContentsMargins(8, 0, 8, 0)
 
         cool_font = QFont()
         cool_font.setWeight(QFont.DemiBold)
@@ -168,8 +229,9 @@ class BadgerOptMonitor(QWidget):
         # btn_edit.setFixedSize(64, 32)
         # btn_edit.setFont(cool_font)
         self.btn_log = btn_log = QPushButton('Logbook')
-        btn_log.setFixedSize(64, 32)
+        btn_log.setFixedSize(128, 32)
         btn_log.setFont(cool_font)
+        btn_log.setStyleSheet(stylesheet_log)
         self.btn_reset = btn_reset = QPushButton('Reset')
         btn_reset.setDisabled(True)
         btn_reset.setFixedSize(64, 32)
@@ -189,6 +251,7 @@ class BadgerOptMonitor(QWidget):
         btn_stop.setDisabled(True)
         btn_stop.setFixedSize(128, 32)
         btn_stop.setFont(cool_font)
+        btn_stop.setStyleSheet(stylesheet_run)
         hbox_action.addWidget(btn_del)
         # hbox_action.addWidget(btn_edit)
         hbox_action.addWidget(btn_log)
@@ -209,6 +272,8 @@ class BadgerOptMonitor(QWidget):
         self.ins_obj.sigPositionChangeFinished.connect(self.ins_drag_done)
         self.ins_con.sigDragged.connect(self.ins_con_dragged)
         self.ins_con.sigPositionChangeFinished.connect(self.ins_drag_done)
+        self.ins_sta.sigDragged.connect(self.ins_sta_dragged)
+        self.ins_sta.sigPositionChangeFinished.connect(self.ins_drag_done)
         self.ins_var.sigDragged.connect(self.ins_var_dragged)
         self.ins_var.sigPositionChangeFinished.connect(self.ins_drag_done)
         self.plot_obj.scene().sigMouseClicked.connect(self.on_mouse_click)
@@ -256,6 +321,10 @@ class BadgerOptMonitor(QWidget):
                 self.con_names = []
         except:
             self.con_names = []
+        try:
+            self.sta_names = self.routine['config']['states'] or []
+        except KeyError:  # this would happen when rerun an old version routine
+            self.sta_names = []
 
         # Configure plots
         # Clear current plots
@@ -266,12 +335,18 @@ class BadgerOptMonitor(QWidget):
             self.plot_con.addItem(self.ins_con)
         except:
             pass
+        try:
+            self.plot_sta.clear()
+            self.plot_sta.addItem(self.ins_sta)
+        except:
+            pass
         self.plot_var.clear()
         self.plot_var.addItem(self.ins_var)
         # Put in the empty curves
         self.curves_var = []
         self.curves_obj = []
         self.curves_con = []
+        self.curves_sta = []
 
         for i, obj_name in enumerate(self.obj_names):
             color = self.colors[i % len(self.colors)]
@@ -317,10 +392,41 @@ class BadgerOptMonitor(QWidget):
                 del self.plot_con
             except:
                 pass
+
+        if self.sta_names:
+            try:
+                self.plot_sta
+            except:
+                self.plot_sta = plot_sta = self.monitor.addPlot(
+                    row=2, col=0, title='Evaluation History (S)')
+                plot_sta.setLabel('left', 'states')
+                plot_sta.setLabel('bottom', 'iterations')
+                plot_sta.showGrid(x=True, y=True)
+                leg_sta = plot_sta.addLegend()
+                leg_sta.setBrush((50, 50, 100, 200))
+                plot_sta.addItem(self.ins_sta)
+                plot_sta.setXLink(self.plot_obj)
+
+            for i, sta_name in enumerate(self.sta_names):
+                color = self.colors[i % len(self.colors)]
+                symbol = self.symbols[i % len(self.colors)]
+                _curve = self.plot_sta.plot(pen=pg.mkPen(color, width=3),
+                                            # symbol=symbol,
+                                            name=sta_name)
+                self.curves_sta.append(_curve)
+        else:
+            try:
+                self.monitor.removeItem(self.plot_sta)
+                self.plot_sta.removeItem(self.ins_sta)
+                del self.plot_sta
+            except:
+                pass
+
         # Reset inspectors
         self.ins_obj.setValue(0)
         self.ins_var.setValue(0)
         self.ins_con.setValue(0)
+        self.ins_sta.setValue(0)
 
         # Switch run button state
         if self.routine:
@@ -328,11 +434,15 @@ class BadgerOptMonitor(QWidget):
         else:
             self.btn_stop.setDisabled(True)
 
+        self.eval_count = 0  # reset the evaluation count
+        self.enable_auto_range()
+
         # Fill in data
         self.data = data
         self.vars = []
         self.objs = []
         self.cons = []
+        self.stas = []
         self.ts = []
         if self.routine_runner and self.routine_runner.run_filename == run_filename:
             self.btn_reset.setDisabled(False)
@@ -344,7 +454,6 @@ class BadgerOptMonitor(QWidget):
             self.btn_del.setDisabled(True)
             self.btn_log.setDisabled(True)
             self.btn_opt.setDisabled(True)
-            self.enable_auto_range()
             return
 
         for var in self.var_names:
@@ -356,9 +465,12 @@ class BadgerOptMonitor(QWidget):
         for con in self.con_names:
             self.cons.append(data[con])
         self.cons = np.array(self.cons).T.tolist()
+        for sta in self.sta_names:
+            self.stas.append(data[sta])
+        self.stas = np.array(self.stas).T.tolist()
         self.ts = data['timestamp_raw']
 
-        self.update_curves(auto_range=True)
+        self.update_curves()
 
         self.calc_optimals()
         self.btn_del.setDisabled(False)
@@ -402,6 +514,7 @@ class BadgerOptMonitor(QWidget):
         self.thread_pool.start(self.routine_runner)
 
         self.btn_stop.setText('Stop')
+        self.btn_stop.setStyleSheet(stylesheet_stop)
         self.btn_ctrl.setDisabled(False)
         self.sig_lock.emit(True)
 
@@ -435,8 +548,10 @@ class BadgerOptMonitor(QWidget):
         self.plot_var.enableAutoRange()
         if self.con_names:
             self.plot_con.enableAutoRange()
+        if self.sta_names:
+            self.plot_sta.enableAutoRange()
 
-    def update_curves(self, auto_range=False):
+    def update_curves(self):
         type_x = self.plot_x_axis
         type_y = self.x_plot_y_axis
         relative = self.x_plot_relative
@@ -473,17 +588,28 @@ class BadgerOptMonitor(QWidget):
                 else:
                     self.curves_con[i].setData(np.array(self.cons)[:, i])
 
-        if auto_range:
-            self.enable_auto_range()
+        if self.sta_names:
+            for i in range(len(self.sta_names)):
+                if type_x:
+                    self.curves_sta[i].setData(ts, np.array(self.stas)[:, i])
+                else:
+                    self.curves_sta[i].setData(np.array(self.stas)[:, i])
 
-    def update(self, vars, objs, cons, ts):
+    def update(self, vars, objs, cons, stas, ts):
         self.vars.append(vars)
         self.objs.append(objs)
         self.cons.append(cons)
+        self.stas.append(stas)
         self.ts.append(ts)
 
-        self.update_curves(auto_range=True)
-        self.sig_progress.emit(vars, objs, cons)
+        self.update_curves()
+
+        # Quick-n-dirty fix to the auto range issue
+        self.eval_count += 1
+        if self.eval_count < 5:
+            self.enable_auto_range()
+
+        self.sig_progress.emit(vars, objs, cons, stas)
 
         # Check critical condition
         critical, msg = self.is_critical(cons)
@@ -512,6 +638,7 @@ class BadgerOptMonitor(QWidget):
         self.btn_ctrl.setText('Pause')
         self.btn_ctrl.setDisabled(True)
         self.btn_stop.setText('Run')
+        self.btn_stop.setStyleSheet(stylesheet_run)
         self.btn_reset.setDisabled(False)
         self.btn_set.setDisabled(False)
         self.btn_del.setDisabled(False)
@@ -519,14 +646,18 @@ class BadgerOptMonitor(QWidget):
         self.sig_lock.emit(False)
 
         try:
-            run = archive_run(self.routine_runner.routine, self.routine_runner.data)
+            run = archive_run(self.routine_runner.routine, self.routine_runner.data,
+                self.routine_runner.states)
             self.routine_runner.run_filename = run['filename']
             self.sig_run_name.emit(run['filename'])
-        except Exception as e:
-            QMessageBox.critical(self, 'Archive failed!', str(e))
 
-        QMessageBox.information(
-            self, 'Success!', f'Run data archived to {BADGER_ARCHIVE_ROOT}')
+            QMessageBox.information(
+                self, 'Success!', f'Archive succeeded: Run data archived to {BADGER_ARCHIVE_ROOT}')
+
+        except Exception as e:
+            self.sig_run_name.emit(None)
+
+            QMessageBox.critical(self, 'Archive failed!', f'Archive failed: {str(e)}')
 
     def on_error(self, error):
         QMessageBox.critical(self, 'Error!', str(error))
@@ -569,15 +700,27 @@ class BadgerOptMonitor(QWidget):
         self.ins_var.setValue(ins_obj.value())
         if self.con_names:
             self.ins_con.setValue(ins_obj.value())
+        if self.sta_names:
+            self.ins_sta.setValue(ins_obj.value())
 
     def ins_con_dragged(self, ins_con):
         self.ins_var.setValue(ins_con.value())
         self.ins_obj.setValue(ins_con.value())
+        if self.sta_names:
+            self.ins_sta.setValue(ins_con.value())
+
+    def ins_sta_dragged(self, ins_sta):
+        self.ins_var.setValue(ins_sta.value())
+        self.ins_obj.setValue(ins_sta.value())
+        if self.con_names:
+            self.ins_con.setValue(ins_sta.value())
 
     def ins_var_dragged(self, ins_var):
         self.ins_obj.setValue(ins_var.value())
         if self.con_names:
             self.ins_con.setValue(ins_var.value())
+        if self.sta_names:
+            self.ins_sta.setValue(ins_var.value())
 
     def ins_drag_done(self, ins):
         self.sync_ins(ins.value())
@@ -590,6 +733,8 @@ class BadgerOptMonitor(QWidget):
         self.ins_obj.setValue(value)
         if self.con_names:
             self.ins_con.setValue(value)
+        if self.sta_names:
+            self.ins_sta.setValue(value)
         self.ins_var.setValue(value)
 
         self.sig_inspect.emit(idx)
@@ -637,6 +782,8 @@ class BadgerOptMonitor(QWidget):
         self.ins_obj.setValue(value)
         if self.con_names:
             self.ins_con.setValue(value)
+        if self.sta_names:
+            self.ins_sta.setValue(value)
         self.ins_var.setValue(value)
 
     def set_vars(self):
@@ -666,11 +813,15 @@ class BadgerOptMonitor(QWidget):
             self.plot_obj.setLabel('bottom', 'time (s)')
             if self.con_names:
                 self.plot_con.setLabel('bottom', 'time (s)')
+            if self.sta_names:
+                self.plot_sta.setLabel('bottom', 'time (s)')
         else:
             self.plot_var.setLabel('bottom', 'iterations')
             self.plot_obj.setLabel('bottom', 'iterations')
             if self.con_names:
                 self.plot_con.setLabel('bottom', 'iterations')
+            if self.sta_names:
+                self.plot_sta.setLabel('bottom', 'iterations')
 
         # Update inspector line position
         if i:
@@ -680,9 +831,12 @@ class BadgerOptMonitor(QWidget):
         self.ins_obj.setValue(value)
         if self.con_names:
             self.ins_con.setValue(value)
+        if self.sta_names:
+            self.ins_sta.setValue(value)
         self.ins_var.setValue(value)
 
-        self.update_curves(auto_range=True)
+        self.update_curves()
+        self.enable_auto_range()
 
     def select_x_plot_y_axis(self, i):
         self.x_plot_y_axis = i
@@ -697,12 +851,16 @@ class BadgerOptMonitor(QWidget):
         coor_obj = self.plot_obj.vb.mapSceneToView(event._scenePos)
         if self.con_names:
             coor_con = self.plot_con.vb.mapSceneToView(event._scenePos)
+        if self.sta_names:
+            coor_sta = self.plot_sta.vb.mapSceneToView(event._scenePos)
         coor_var = self.plot_var.vb.mapSceneToView(event._scenePos)
 
         flag = self.plot_obj.viewRect().contains(coor_obj) or \
             self.plot_var.viewRect().contains(coor_var)
         if self.con_names:
             flag = flag or self.plot_con.viewRect().contains(coor_con)
+        if self.sta_names:
+            flag = flag or self.plot_sta.viewRect().contains(coor_sta)
 
         if flag:
             self.sync_ins(coor_obj.x())
